<?
   if(!isset($GLOBALS[__FILE__]))
   {
      $GLOBALS[__FILE__] = true;

      define("SVG_NONE",  "none");
      define("SVG_BLACK", "black");

      error_reporting(-1);

      // ------------------------------------
      // KV
      // ------------------------------------

      class KV
      {
         function __construct($key, $value)
         {
            $this->mKey   = $key;
            $this->mValue = $value;
         }

         var $mKey;
         var $mValue;
      }

      // ------------------------------------
      // Tag
      // ------------------------------------

      class Tag
      {
         function __construct($name)
         {
            $this->mName = $name;
            $this->mKV   = array();
         }

         function set($key, $value)
         {
            $this->mKV[$key] = $value;
         }

         function toString()
         {
            $s = "<" . $this->mName;

            foreach($this->mKV as $key => $value)
               $s .= " " . sprintf("%s=\"%s\"", $key, $value);

            return($s . "/>");
         }

         var $mName;
         var $mKV;
      }

      // ------------------------------------
      // SVG
      // ------------------------------------

      class SVG
      {
         // ----------------
         // Constructor
         // ----------------

         function __construct($width, $height)
         {
            $this->mWidth  = $width;
            $this->mHeight = $height;
            $this->mTags   = array();
         }

         // ----------------
         // toString
         // ----------------

         function toString()
         {
            $s = "";

            foreach($this->mTags as $tag)
               $s .= sprintf("%6s%s\n", "", $tag->toString());

            return(
               SVG::getHeader($this->mWidth, $this->mHeight) .
               $s .
               SVG::getFooter()
            );
         }

         // ----------------
         // line
         // ----------------

         function line($x1, $y1, $x2, $y2, $stroke)
         {
            $t = new Tag("line");

            $t->set("x1",     $x1);
            $t->set("y1",     $y1);
            $t->set("x2",     $x2);
            $t->set("y2",     $y2);
            $t->set("stroke", $stroke);

            $this->mTags[] = $t;
         }

         // ----------------
         // rect
         // ----------------

         function rect($x, $y, $width, $height, $stroke, $fill)
         {
            $t = new Tag("rect");

            $t->set("x",      $x);
            $t->set("y",      $y);
            $t->set("width",  $width);
            $t->set("height", $height);
            $t->set("stroke", $stroke);
            $t->set("fill",   $fill);

            $this->mTags[] = $t;
         }

         // ----------------
         // circle
         // ----------------

         function circle($x, $y, $radius, $stroke, $fill)
         {
            $t = new Tag("circle");

            $t->set("cx",     $x);
            $t->set("cy",     $y);
            $t->set("r",      $radius);
            $t->set("fill",   $fill);
            $t->set("stroke", $stroke);

            $this->mTags[] = $t;
         }

         // ----------------
         // border
         // ----------------

         function border($color)
         {
            $this->rect(0, 0, $this->mWidth, $this->mHeight, $color, SVG_NONE);
         }

         // ----------------
         // toHTML
         // ----------------

         static function toHTML($svg, $title = "SVG Output")
         {
            $asString = $svg->toString();

            $html = <<< EOF
<html>
 <head><title>$title</title></head>
 <body>
  $asString
 </body>
</html>
EOF;

            return($html);
         }

         // ----------------
         // getHeader
         // ----------------

         static function getHeader($width, $height)
         {
            $header = <<< EOF
   <svg
      xmlns="http://www.w3.org/2000/svg"
      xmlns:xlink="http://www.w3.org/1999/xlink"
      width="$width" height="$height">

EOF;

            return($header);
         }

         // ----------------
         // getFooter
         // ----------------

         static function getFooter()
         {
            return("   </svg>");
         }

         // ----------------
         // Member Variables
         // ----------------

         var $mWidth;
         var $mHeight;
         var $mTags;
      }
   }
?>
